import 'package:json_api_document/src/attributes.dart';
import 'package:json_api_document/src/link.dart';
import 'package:json_api_document/src/meta.dart';
import 'package:json_api_document/src/naming.dart';
import 'package:json_api_document/src/relationship.dart';

class Resource {
  final String type;
  final String id;
  final Attributes attributes;
  final Link self;
  final Meta meta;
  final Map<String, Relationship> relationships;

  Resource(String this.type, String this.id,
      {Map<String, dynamic> attributes,
      Link this.self,
      Map<String, dynamic> meta,
      Map<String, Relationship> relationships})
      : meta = Meta.fromJson(meta),
        attributes = Attributes.fromJson(attributes),
        relationships =
            relationships == null ? null : Map.unmodifiable(relationships) {
    if (id != null && id.isEmpty) throw ArgumentError();
    final naming = const Naming();
    naming.enforce(type);
    if (relationships != null) {
      relationships.keys.forEach((String attr) {
        naming.enforce(attr);
        if (['type', 'id'].contains(attr)) throw ArgumentError();
      });

      if (attributes != null) {
        final fields = Set<String>.of(attributes.keys);
        final unique = relationships.keys.every(fields.add);
        if (!unique) throw ArgumentError();
      }
    }
  }

  toJson() {
    final Map<String, dynamic> j = {'type': type};
    if (id != null) j['id'] = id;
    if (attributes != null) j['attributes'] = attributes;
    if (meta != null) j['meta'] = meta;
    if (self != null) j['links'] = {'self': self};
    if (relationships != null && relationships.isNotEmpty)
      j['relationships'] = relationships;

    return j;
  }

  bool identifies(Resource resource) =>
      relationships != null &&
      relationships.values.any((rel) => rel.identifies(resource));
}
